/***************************************************************************//**
 *   @file   Main.c
 *   @brief  Implementation of the program's main function.
 *   @author Bancisor Mihai
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 743
*******************************************************************************/

/******************************************************************************/
/***************************** Include Files **********************************/
/******************************************************************************/
#include "Communication.h"
#include "ADP5589.h"

/******************************************************************************/
/*********************** Device configuration words ***************************/
/******************************************************************************/
#pragma config POSCMOD  = XT        /*!< XT oscillator mode selected */
#pragma config FNOSC    = PRIPLL    /*!< Primary Oscillator with PLL Module*/
#pragma config FPLLMUL  = MUL_20    /*!< 20x multiplier */
#pragma config FPLLIDIV = DIV_2     /*!< 2x divider */
#pragma config FPLLODIV = DIV_1     /*!< PLL output divided by 1 */
#pragma config FPBDIV   = DIV_1     /*!< PBCLK is SYSCLK divided by 1 */
#pragma config FWDTEN   = OFF       /*!< The WDT is not enabled */

/******************************************************************************/
/************************ Constants Definitions *******************************/
/******************************************************************************/
/*! List of available commands */
const char* commandsList[] = {"help?",
                              "pwmOnTime=",
                              "pwmOffTime=",
                              "lastKey?"
                               };
const char* commandsDescription[] = {
    "  -  Displays all available commands.",
    "  -  On time of the PWM pulse. Accepted values: 0 - 65535",
    "  -  Off time of the PWM pulse. Accepted values: 0 - 65535",
    "  -  Displays the last key which was pressed"};
/******************************************************************************/
/************************ Variables Definitions *******************************/
/******************************************************************************/
unsigned char commandsNumber = (sizeof(commandsList) / sizeof(const char*));
unsigned char receivedCommand[20];
unsigned char invalidCommand = 0;
unsigned char commandType = 0;
unsigned char command = 0;
unsigned char displayCommand = 0;
unsigned short commandParam = 0;

/***************************************************************************//**
 * @brief Main function.
 *
 * @return None.
*******************************************************************************/
void main(void)
{
    /*! Temporary variables */
    char  tempString [10]  = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
    /*! Variables holding information about the device */
    unsigned short pwmOnTime  = 1;
    unsigned short pwmOffTime   = 1;
    char key = 0;
    char keyTemp = 0;
    char lastKey = 0;
    unsigned char ct = 0;
    
    /*! Initialize the UART communication peripheral. */
    UART_Init(9600);
    if(ADP5589_Init())
    {
        CONSOLE_WriteString("ADP5589 OK\n");
    }
    else
    {
        CONSOLE_WriteString("ADP5589 ERR\n");
    }
    ADP5589_InitKey(PMOD_IOXP_J2);	// Init ADP5589 as keyboard decoder
    ADP5589_InitPwm();                  // Init PWM on pin R3
    ADP5589_SetPwm(1,1);
    while(1)
    {
         CONSOLE_GetCommand(receivedCommand);
        invalidCommand = 0;
        for(command = 0; command < commandsNumber; command++)
        {
            commandType = CONSOLE_CheckCommands(receivedCommand,
                                                commandsList[command],
                                                &commandParam);
            if(commandType == 0)
            {
                invalidCommand++;
            }
            if((command == 0) && (commandType != 0))    /*!< "help?" command*/
            {
                CONSOLE_WriteString("Available commands:");
                UART_Write(0x0D);
                for(displayCommand = 0; displayCommand < commandsNumber;
                    displayCommand++)
                {
                    CONSOLE_WriteString(commandsList[displayCommand]);
                    CONSOLE_WriteString(commandsDescription[displayCommand]);
                    UART_Write(0x0D);
                }
            }
            if((command == 1) && (commandType != 0))  /*!< "pwmOnTime=" command*/
            {
                /*! Check if the parameter is valid */
                if( (commandParam >= 0) && (commandParam <= 0xFFFF) )
                {
                    pwmOnTime = (unsigned short)commandParam;
                }
                else if(commandParam < 0)
                {
                    pwmOnTime = 0;
                }
                else
                {
                    pwmOnTime = 0xFFFF;
                }
                /*! Configure the PWM on/off registers */
                ADP5589_SetPwm(pwmOffTime, pwmOnTime);
                /*! Send feedback to user */
                CONSOLE_WriteString(commandsList[command]);
               // pwmOnTime = 13;
                itoa(tempString, pwmOnTime, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 2) && (commandType != 0))  /*!< "pwmOffTime=" command*/
            {
                /*! Check if the parameter is valid */
                if( (commandParam >= 0) && (commandParam <= 0xFFFF) )
                {
                    pwmOffTime = commandParam;
                }
                else if(commandParam < 0)
                {
                    pwmOffTime = 0;
                }
                else
                {
                    pwmOffTime = 0xFFFF;
                }
                /*! Configure the PWM on/off registers */
                ADP5589_SetPwm(pwmOffTime, pwmOnTime);
                /*! Send feedback to user */
                CONSOLE_WriteString(commandsList[command]);
                //sprintf(tempString, "%u", pwmOffTime);
                itoa(tempString, pwmOffTime, 10);
                CONSOLE_WriteString(tempString);
                UART_Write(0xD);
            }
            if((command == 3) && (commandType != 0))   /*!< "lastKey?" command*/
            {
                /*! Send the requested value to user */
                for(ct = 0; ct < 16; ct++)
                {
                    keyTemp = ADP5589_GetRegisterValue(ADP5589_ADR_FIFO1);
                    key = ADP5589_KeyDecode(keyTemp,PMOD_IOXP_J2);
                    if (key != 'x')
                    {
                        lastKey = key;
                    }
                }
                        CONSOLE_WriteString("Last key press: ");
                        UART_Write(lastKey);
                        UART_Write(0xD);
            }
        }
    }
}
